import pandas as pd
import matplotlib.lines as lines


class CalcTop5Class:
    def __init__(self):
        self._data = []
        self._top5 = []
        self._dropped = []

    def count_top5(self, target_row):
        """
        Calculate top 5 data
        """
        self._data = target_row.value_counts(sort=True) / target_row.value_counts().sum() * 100
        self._top5 = self._data[:5]
        self._dropped = self._data.drop(self._top5.index)
        self._top5["(Others)"] = pd.Series(self._dropped[0:].sum(), dtype="float64").to_string(index=False)
        return self._top5


class DrawpieChartClass:
    def __init__(self):
        self._colors = ["#2466a9", "#3f86bf", "#609fc6", "#88b5ce", "#bcd4e5", "#B2B2B2"]  # Blue palette
        self._wedgeprops = {"alpha": 0.9, "edgecolor": "white", "linewidth": 1, "width": 0.7}
        self._textprops = {"weight": "bold", "color": "#ffff00", "fontsize": "14"}

    def pie_chart(self, ax, data, title):
        """
        Draw pie chart
        """
        ax.pie(data, colors=self._colors, startangle=90, counterclock=False,
               labels=data.index, pctdistance=0.7,
               autopct=lambda p: "{: .0f}%".format(p) if p >= 3 else "",
               wedgeprops=self._wedgeprops, textprops=self._textprops, labeldistance=None)
        ax.legend(loc="upper left", bbox_to_anchor=(1.3, 0.9),
                  prop={"size": "16", "weight": "bold"}, frameon=False, ncol=1)
        ax.set_title(title, loc="left", pad=10, fontsize="20", fontweight="bold", color="black")


def draw_pie_chart(data_frame, fig_a, fig_b, mask_ip):
    """
    Draw pie chart for top 5
    """
    global top5
    top5 = [0] * 4

    calc = CalcTop5Class()

    for i in range(len(top5)):
        top5[i] = calc.count_top5(data_frame.iloc[:, i + 1])

    if mask_ip:
        top5[2] = (mask_ip_addresses(top5[2]))

    fig_a.subplots_adjust(top=0.8, left=-0.5, hspace=0.2)
    fig_b.subplots_adjust(top=0.8, left=-0.5, hspace=0.2)

    # Add note for country code
    fig_b.add_artist(lines.Line2D([0.66, 0.93], [0.16, 0.16], color="#eeeeee", linewidth=60, zorder=0))
    cc_note = "Please refer to the following for the country code."
    cc_link = "https://en.wikipedia.org/wiki/ISO_3166-2"
    fig_b.text(0.64, 0.17, cc_note, fontsize=12, ha="left")
    fig_b.text(0.64, 0.13, cc_link, fontsize=12, ha="left", color="blue",
               url="https://en.wikipedia.org/wiki/ISO_3166-2")

    ax_a_1 = fig_a.add_subplot(2, 1, 1)
    ax_a_2 = fig_a.add_subplot(2, 1, 2)
    ax_b_1 = fig_b.add_subplot(2, 1, 1)
    ax_b_2 = fig_b.add_subplot(2, 1, 2)

    ax_a_1.set_position([0.004, 0.42, 0.4, 0.3])
    ax_a_2.set_position([0.004, 0.06, 0.4, 0.3])
    ax_b_1.set_position([0.004, 0.42, 0.4, 0.3])
    ax_b_2.set_position([0.004, 0.06, 0.4, 0.3])

    draw = DrawpieChartClass()

    draw.pie_chart(ax_a_1, top5[0], "Terminating Rule Group")
    draw.pie_chart(ax_a_2, top5[1], "Path Name")
    draw.pie_chart(ax_b_1, top5[2], "IP Address")
    draw.pie_chart(ax_b_2, top5[3], "Country")


def mask_ip_addresses(ip_dataframe):
    """
    Mask IP adress if the option was set
    """
    masked_ips = []
    for i in range(len(ip_dataframe)):
        ip_segments = str(ip_dataframe.index[i]).split(".")
        if i != 5:
            ip_segments[0] = "xxx"
            ip_segments[1] = "xxx"
        masked_ip = '.'.join(ip_segments)
        masked_ips.append(masked_ip)
    ip_dataframe.index = [masked_ips[0], masked_ips[1], masked_ips[2],
                          masked_ips[3], masked_ips[4], masked_ips[5]]
    return ip_dataframe
